
#include <sys/param.h>
#include "stand.h"

struct open_file files[SOPEN_MAX];

void
files_init (void)
{
  memset (files, 0, sizeof (files));
}

int
open (const char *fname, int mode)
{
  struct open_file *f;
  int fd, error;

  char *file;

  printf ("open %s start\n", fname);

  /* find a free file descriptor */
  for (fd = 0, f = files; fd < SOPEN_MAX; fd++, f++)
  {
    if (f->f_flags == 0)
      goto fnd;
  }
  printf ("open %s failed to get  free file descriptor\n", fname);
  errno = EMFILE;
  return (-1);

fnd:
  /*Try to open the device. Convert open mode (0,1,2) to F_READ, F_WRITE. */
  f->f_flags = mode + 1;
  f->f_dev = (struct devsw *) 0;
  f->f_offset = 0;
  file = (char *) 0;
  error = devopen (f, fname, &file);
  if (error || (((f->f_flags & F_NODEV) == 0) && f->f_dev == (struct devsw *) 0))
    goto err;

  /* see if we opened a raw device; otherwise, 'file' is the file name. */
  if (file == (char *) 0 || *file == '\0')
  {
    f->f_flags |= F_RAW;
    return (fd);
  }

  /* pass file name to the different filesystem open routines */
  error = mem_open (file, f);
  if (error == 0)
    return (fd);

  if (error == EINVAL)
    error = ENOENT;

  if ((f->f_flags & F_NODEV) == 0)
    (void) f->f_dev->dv_close (f);

err:
  f->f_flags = 0;
  errno = error;
  return (-1);
}

int
close (int fd)
{
  struct open_file *f = &files[fd];
  int err1 = 0, err2 = 0;

  if ((unsigned) fd >= SOPEN_MAX || f->f_flags == 0)
  {
    errno = EBADF;
    return (-1);
  }

  if (!(f->f_flags & F_RAW))
    err1 = mem_close (f);
  if (!(f->f_flags & F_NODEV))
    if (f->f_dev != NULL)
      err2 = (f->f_dev->dv_close) (f);
  f->f_flags = 0;
  if (err1)
  {
    errno = err1;
    return (-1);
  }
  if (err2)
  {
    errno = err2;
    return (-1);
  }
  return (0);
}

off_t
lseek (int fd, off_t offset, int where)
{
  struct open_file *f = &files[fd];

  if ((unsigned) fd >= SOPEN_MAX || f->f_flags == 0)
  {
    errno = EBADF;
    return (-1);
  }

  if (f->f_flags & F_RAW)
  {
    /*
     * On RAW devices, update internal offset.
     */
    switch (where)
    {
    case SEEK_SET:
      f->f_offset = offset;
      break;
    case SEEK_CUR:
      f->f_offset += offset;
      break;
    case SEEK_END:
    default:
      errno = EOFFSET;
      return (-1);
    }
    return (f->f_offset);
  }

  return mem_seek (f, offset, where);
}

ssize_t
read (int fd, void *dest, size_t bcount)
{
  struct open_file *f = &files[fd];
  size_t resid;

  if ((unsigned) fd >= SOPEN_MAX || !(f->f_flags & F_READ))
  {
    errno = EBADF;
    return (-1);
  }

  if (f->f_flags & F_RAW)
  {
    errno = f->f_dev->dv_strategy (f->f_devdata, F_READ, btodb (f->f_offset), bcount, dest, &resid);
    if (errno)
      return (-1);
    f->f_offset += resid;
    return (resid);
  }
  resid = bcount;
  if ((errno = mem_read (f, dest, bcount, &resid)))
    return (-1);
  return (ssize_t) (bcount - resid);
}
