
#include <stdio.h>
#include <stdarg.h>

void panic (char *fmt, ...);

void
panic (char *fmt, ...)
{
  char buffer[256];
  va_list ap;

  va_start (ap, fmt);
  vsnprintf (buffer, 256, fmt, ap);
  va_end (ap);

  puts (buffer);
  while (1);
}

int
printf (const char *fmt, ...)
{
  char buffer[256];
  va_list ap;

  va_start (ap, fmt);
  vsnprintf (buffer, 256, fmt, ap);
  va_end (ap);

  puts (buffer);
  return 0;
}

char *
strchr (const char *s, int c)
{
  while (*s != '\0' && *s != c)
    s++;
  return ((*s == c) ? (char *) s : NULL);
}

char *
strcpy (char *s1, const char *s2)
{
  char *dst = s1;
  const char *src = s2;

  /* Do the copying in a loop.  */
  while ((*dst++ = *src++) != '\0')
    ;
  /* Return the destination string.  */
  return s1;
}

int
strncmp (const char *s1, const char *s2, size_t n)
{
  unsigned char uc1, uc2;

  /* Nothing to compare?  Return zero.  */
  if (n == 0)
    return 0;
  /* Loop, comparing bytes.  */
  while (n-- > 0 && *s1 == *s2)
  {
    /* If we've run out of bytes or hit a null, return zero
       since we already know *s1 == *s2.  */
    if (n == 0 || *s1 == '\0')
      return 0;
    s1++;
    s2++;
  }
  uc1 = (*(unsigned char *) s1);
  uc2 = (*(unsigned char *) s2);
  return ((uc1 < uc2) ? -1 : (uc1 > uc2));
}

void *
memset (void *s, int c, size_t n)
{
  unsigned char *us = s;
  unsigned char uc = c;

  while (n-- != 0)
    *us++ = uc;
  return s;
}

void *
memcpy (void *s1, const void *s2, size_t n)
{
  char *dst = s1;
  const char *src = s2;

  while (n-- != 0)
    *dst++ = *src++;
  return s1;
}

int
memcmp (const void *s1, const void *s2, size_t n)
{
  unsigned char *us1 = (unsigned char *) s1;
  unsigned char *us2 = (unsigned char *) s2;

  while (n-- != 0)
  {
    if (*us1 != *us2)
      return (*us1 < *us2) ? -1 : +1;
    us1++;
    us2++;
  }
  return 0;
}

int
snprintf (char *buf, size_t size, const char *fmt, ...)
{
  va_list ap;
  int len;

  va_start (ap, fmt);
  len = vsnprintf (buf, size, fmt, ap);
  va_end (ap);
  return (len);
}

extern char end[];
static char *top = (char *) end;

#define ALIGN(x) (((unsigned long) (sizeof (x)) + (sizeof (double) - 1)) & ~(sizeof (double) - 1))

void *
alloc (unsigned int size)
{
  char *help;

  help = top;
  top += ALIGN (sizeof (unsigned)) + ALIGN (size);
  printf ("----------------> alloc = 0x%x size = %d\n", help + ALIGN (sizeof (unsigned)), size);
  return (help + ALIGN (sizeof (unsigned)));
}

void
free (void *ptr, unsigned int size)
{
}
