
/*
**  [subsequent definitions are from image.h]
*/

/*
 * Operating System Codes
 */
#define	IH_OS_INVALID		0       /* Invalid OS   */
#define	IH_OS_OPENBSD		1       /* OpenBSD      */
#define	IH_OS_NETBSD		2       /* NetBSD       */
#define	IH_OS_FREEBSD		3       /* FreeBSD      */
#define	IH_OS_4_4BSD		4       /* 4.4BSD       */
#define	IH_OS_LINUX		5       /* Linux        */
#define	IH_OS_SVR4		6       /* SVR4         */
#define	IH_OS_ESIX		7       /* Esix         */
#define	IH_OS_SOLARIS		8       /* Solaris      */
#define	IH_OS_IRIX		9       /* Irix         */
#define	IH_OS_SCO		10      /* SCO          */
#define	IH_OS_DELL		11      /* Dell         */
#define	IH_OS_NCR		12      /* NCR          */
#define	IH_OS_LYNXOS		13      /* LynxOS       */
#define	IH_OS_VXWORKS		14      /* VxWorks      */
#define	IH_OS_PSOS		15      /* pSOS         */
#define	IH_OS_QNX		16      /* QNX          */

/*
 * CPU Architecture Codes (supported by Linux)
 */
#define	IH_CPU_INVALID		0       /* Invalid CPU  */
#define	IH_CPU_ALPHA		1       /* Alpha        */
#define	IH_CPU_ARM		2       /* ARM          */
#define	IH_CPU_I386		3       /* Intel x86    */
#define	IH_CPU_IA64		4       /* IA64         */
#define	IH_CPU_MIPS		5       /* MIPS         */
#define	IH_CPU_MIPS64		6       /* MIPS  64 Bit */
#define	IH_CPU_PPC		7       /* PowerPC      */
#define	IH_CPU_S390		8       /* IBM S390     */
#define	IH_CPU_SH		9       /* SuperH       */
#define	IH_CPU_SPARC		10      /* Sparc        */
#define	IH_CPU_SPARC64		11      /* Sparc 64 Bit */

#define	IH_TYPE_INVALID		0       /* Invalid Image                */
#define IH_TYPE_STANDALONE	1       /* Standalone Program           */
#define IH_TYPE_KERNEL		2       /* OS Kernel Image              */
#define IH_TYPE_RAMDISK		3       /* RAMDisk Image                */
#define IH_TYPE_MULTI		4       /* Multi-File Image             */

/*
 * Compression Types
 */
#define IH_COMP_NONE		0       /*  No   Compression Used       */
#define IH_COMP_GZIP		1       /* gzip  Compression Used       */
#define IH_COMP_BZIP2		2       /* bzip2 Compression Used       */

#define	IH_MAGIC	0x27051956      /* Image Magic Number           */
#define	IH_NMLEN		32      /* Image Name Length            */

typedef void (interrupt_handler_t) (void *);

typedef struct monitor_functions
{
  int (*getc) (void);
  int (*tstc) (void);
  void (*putc) (const char c);
  void (*puts) (const char *s);
  void (*printf) (const char *fmt, ...);
  void (*install_hdlr) (int, interrupt_handler_t *, void *);
  void (*free_hdlr) (int);
  void *(*malloc) (size_t);
  void (*free) (void *);
  void (*install_bsp_hdlr) (int, void *, void *);
} mon_fnc_t;

typedef struct bd_info
{
  unsigned long bi_memstart;
  unsigned long bi_memsize;
  unsigned long bi_flashstart;
  unsigned long bi_flashsize;
  unsigned long bi_flashoffset;
  unsigned long bi_sramstart;
  unsigned long bi_sramsize;

  unsigned long bi_bootflags;
  unsigned long bi_ip_addr;
  unsigned char bi_enetaddr[6];
  unsigned short bi_ethspeed;
  unsigned long bi_intfreq;
  unsigned long bi_busfreq;

  unsigned long bi_baudrate;

  unsigned char bi_s_version[4];
  unsigned char bi_r_version[32];
  unsigned int bi_procfreq;
  unsigned int bi_plb_busfreq;
  unsigned int bi_pci_busfreq;
  unsigned char bi_pci_enetaddr[6];

  mon_fnc_t *bi_mon_fnc;
} bd_t;

typedef struct image_header
{
  unsigned long ih_magic;
  unsigned long ih_hcrc;
  unsigned long ih_time;
  unsigned long ih_size;
  unsigned long ih_load;
  unsigned long ih_ep;
  unsigned long ih_dcrc;
  unsigned char ih_os;
  unsigned char ih_arch;
  unsigned char ih_type;
  unsigned char ih_comp;
  unsigned char ih_name[32];
} image_header_t;

#ifndef NULL
#define NULL (void *)0
#endif

bd_t *boot_info;

#define PRINT boot_info->bi_mon_fnc->printf
//#define PRINT printf
#define DPRINT(fmt,...)

#if 0
extern char __start[];
static char *code_start = (char *) __start;
#endif

void boot_puts (char *s);
void boot_main (bd_t * bd, image_header_t * img, char *cons_dev, char *bootargs);
void print_buffer (char *id, char *buf, int len);
void boot_putchar (int c);
void boot_banner (void);
void boot_main1 (bd_t * bd);
void boot_main2 (image_header_t * img);
void boot_main3 (char *cons_dev, char *bootargs);

void
boot_banner (void)
{
  PRINT ("----------------- ppcboot banner ---------------------\n");
}

void
boot_main1 (bd_t * bd)
{
  boot_info = bd;

  PRINT ("NetBSD boot_main\n");

  PRINT ("bd = 0x%x\n", (int) bd);
  PRINT (" bi_memstart = 0x%x\n", (int) bd->bi_memstart);
  PRINT (" bi_memsize = 0x%x\n", (int) bd->bi_memsize);
  PRINT (" bi_flashstart = 0x%x\n", (int) bd->bi_flashstart);
  PRINT (" bi_flashsize = 0x%x\n", (int) bd->bi_flashsize);
  PRINT (" bi_flashoffset = 0x%x\n", (int) bd->bi_flashoffset);
  PRINT (" bi_sramstart = 0x%x\n", (int) bd->bi_sramstart);
  PRINT (" bi_sramsize = 0x%x\n", (int) bd->bi_sramsize);
  PRINT (" bi_bootflags = 0x%x\n", (int) bd->bi_bootflags);
  PRINT (" bi_ip_addr = 0x%x\n", (int) bd->bi_ip_addr);

  PRINT (" bi_ethspeed = 0x%x\n", (int) bd->bi_ethspeed);
  PRINT (" bi_intfreq = 0x%x\n", (int) bd->bi_intfreq);
  PRINT (" bi_busfreq = 0x%x\n", (int) bd->bi_busfreq);
  PRINT (" bi_baudrate = 0x%x\n", (int) bd->bi_baudrate);
  PRINT (" bi_procfreq = 0x%x\n", (int) bd->bi_procfreq);
  PRINT (" bi_plb_busfreq = 0x%x\n", (int) bd->bi_plb_busfreq);
  PRINT (" bi_pci_busfreq = 0x%x\n", (int) bd->bi_pci_busfreq);

  PRINT ("bd->bi_mon_fnc = 0x%x\n", (int) bd->bi_mon_fnc);
  PRINT ("bd->bi_mon_fnc->puts = 0x%x\n", (int) bd->bi_mon_fnc->puts);
  PRINT ("bd->bi_mon_fnc->putc = 0x%x\n", (int) bd->bi_mon_fnc->putc);
  PRINT ("bd->bi_mon_fnc->printf = 0x%x\n", (int) bd->bi_mon_fnc->printf);

}

void
boot_main2 (image_header_t * img)
{
  PRINT ("img = 0x%x\n", (int) img);
  PRINT (" ih_magic = 0x%x\n", (int) img->ih_magic);
  PRINT (" ih_hcrc = 0x%x\n", (int) img->ih_hcrc);
  PRINT (" ih_time = 0x%x\n", (int) img->ih_time);
  PRINT (" ih_size = 0x%x\n", (int) img->ih_size);
  PRINT (" ih_load = 0x%x\n", (int) img->ih_load);
  PRINT (" ih_ep = 0x%x\n", (int) img->ih_ep);
  PRINT (" ih_dcrc = 0x%x\n", (int) img->ih_dcrc);
  PRINT (" ih_os = 0x%x\n", (int) img->ih_os);
  PRINT (" ih_arch = 0x%x\n", (int) img->ih_arch);
  PRINT (" ih_type = 0x%x\n", (int) img->ih_type);
  PRINT (" ih_comp = 0x%x\n", (int) img->ih_comp);
  PRINT (" ih_name = %s\n", img->ih_name);
}

void
boot_main3 (char *cons_dev, char *bootargs)
{
  if (cons_dev)
    PRINT ("cons_dev = %s\n", cons_dev);
  if (bootargs)
    PRINT ("bootargs = %s\n", bootargs);
}

void
boot_main (bd_t * bd, image_header_t * img, char *cons_dev, char *bootargs)
{
  boot_main1 (bd);
  boot_main2 (img);
  boot_main3 (cons_dev, bootargs);

#if 0
  PRINT ("-----------------DUMPING BOOT LOADER CODE START-----------------------\n");
  print_buffer ("", code_start, 64);
  PRINT ("-----------------DUMPING BOOT LOADER CODE END-----------------------\n");
#endif

  PRINT ("exit from PPCboot\n");
}

void
boot_puts (char *s)
{
  if (boot_info == NULL)
    return;
  boot_info->bi_mon_fnc->puts (s);
}

void
boot_putchar (int c)
{
  if (boot_info == NULL)
    return;
  boot_info->bi_mon_fnc->putc (c);
}

void *boot_alloc (unsigned int size);
void *
boot_alloc (unsigned int size)
{
  void *ptr;

  if (boot_info == NULL)
    return NULL;
  ptr = boot_info->bi_mon_fnc->malloc (size);
  DPRINT ("----------------> alloc = 0x%x size = %d\n", ptr, size);
  return ptr;
}

void boot_free (void *ptr, unsigned int size);
void
boot_free (void *ptr, unsigned int size)
{
  if (boot_info == NULL)
    return;
  DPRINT ("----------------> free 0x%x size 0x%x\n", ptr, size);
  boot_info->bi_mon_fnc->free (ptr);
}

void
print_buffer (char *id, char *buf, int len)
{
  int i;

  PRINT ("%s\n", id);
  for (i = 0; i < len; i++)
  {
    if (buf[i] <= 0x0F)
      boot_putchar ('0');

    PRINT ("%x ", buf[i]);

    if (i % 16 == 15)
      boot_putchar ('\n');

  }
  PRINT ("\n");
}
